#include <stdio.h>
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "dat.h"
#include "fns.h"
#include "config.h"

typedef struct {
	Client *cli[MAX_WORKSPACECLIENTS];
} WORKSPACE;

static int current_workspace = 0;
static WORKSPACE map[MAX_WORKSPACE] = { { { NULL } } };

void workspace_delete( Client *c ) {
	int i, j, t;
	for( i=0; i<MAX_WORKSPACE; i++ ) {
		for( j=0; j<MAX_WORKSPACECLIENTS; j++ ) {
			if( map[ i ].cli[ j ] == NULL )
				break;
			if( map[ i ].cli[ j ] == c ) {
				for( t=j+1; t<MAX_WORKSPACECLIENTS; t++ ) {
					map[i].cli[t-1] = map[i].cli[t];
				}
				break;
			}
		}
	}
}

int workspace_mapped( Client *c ) {
	int i, j, t;
	for( i=0; i<MAX_WORKSPACE; i++ ) {
		for( j=0; j<MAX_WORKSPACECLIENTS; j++ ) {
			if( map[ i ].cli[ j ] == NULL )
				break;
			if( map[ i ].cli[ j ] == c ) {
				return( 1 );
			}
		}
	}
	return( 0 );
}

void workspace_hide( Client *c ) {
	if( !c )
		return;
	XUnmapWindow( dpy, c->parent );
	XUnmapWindow( dpy, c->window );
	mysetstate( c, WithdrawnState );
	if( c == current )
		nofocus();
}

void workspace_unhide( Client *c ) {
	if( !c )
		return;
	XMapWindow( dpy, c->window );
	XMapRaised( dpy, c->parent );
	mysetstate( c, NormalState );
	active( c );
	/* We do not call top here. It's unhide that calls top. */
	/* top( c ); */
}

void set_workspace( int new_workspace ) {
	Client *c;
	int i;

	if( ( new_workspace >= MAX_WORKSPACE )
	    ||( new_workspace < 0 )
	    ||( new_workspace == current_workspace ) )
		return;

	/* Save old Workspace */
	for( i = 0, c = clients; c; c = c->next ) {
		if( normal( c ) ) {
			/* Memorize Old WorkSpace Clients */
			if( i < (MAX_WORKSPACECLIENTS-1) ) {
				map[ current_workspace ].cli[ i++ ] = c;
				workspace_hide( c );
			}
		}
	}
	map[ current_workspace ].cli[ i ] = NULL;
	current_workspace = new_workspace;
	/* Get New WorkSpace Clients */
	for( i = 0; map[ current_workspace ].cli[ i ]; i++ ) {
		c = map[ current_workspace ].cli[ i ];
		workspace_unhide( c );
	}
}

void prev_workspace( void ) {
	int wp = (current_workspace - 1);
	if( wp < 0 )
		wp = MAX_WORKSPACE - 1;
	set_workspace( wp );
}

void next_workspace( void ) {
	int wp = (current_workspace + 1);
	if( wp >= MAX_WORKSPACE )
		wp = 0;
	set_workspace( wp );
}
